#!/usr/bin/perl -w

######################################################
#
#	Breakdown wrapper
#
#	The Breakdown wrapper has the ability to produce a Stacked Histogram
#	of the main component of the TCO (Infrastructure, Server, Power, Network and Maintenance)
#	for different configurations
#
#	The results are printed in a file <output_file_name>.txt
#	and the graph is plot in a file <output_file_name>.eps
#
#	How to use it:
#	./breakdown.pl		<dc config file>								datacenter configuration file
#						<resource config file>							resource configuration file
#						<output_file_name>								filename of the output file without extension (the extensions are .txt and .eps)
#						
#						
#						[<dc config file> <resource config file>]		Optional: other pairs of: datacenter configuration file ; resource configuration file
#
#	Requirement: Gnuplot
#
#
#  Example of command:
# ./breakdown.pl ../template/dc_template.params ../template/r_template.params tco_breakdown ../template/dc2_template.params template/r2_template.params
#
######################################################


use strict;

use POSIX qw/ceil/;

use File::Copy;

use lib "../lib/"; 
use helper;


######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 3 || $argv_size%2==0)
{
	print "usage: ./breakdown.pl <dc config file> <resource config file> <output_file_name> [<dc config file 2> <resource config file 2>]...[<dc config file n> <resource config file n>]\n";
	exit(0);
}

my $dc_filename=$ARGV[0];
my $resource_filename=$ARGV[1];

my $filename= $ARGV[2];

######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#open outputfile
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;


######################################################
#
# Breakdown exploration
#
######################################################

my $nbconfig=($argv_size-3)/2;
print(OUTPUTFILE "configs\tInfrastructure\tServer\tPower\tNetwork\tMaintenance\n");


#first configuration
my $ptr_dc=$helper->parserDCFile("$dc_filename");
my $ptr_resource=$helper->parserResourceFile("$resource_filename");

$helper->createParameterFile("$currentdir/dc.params",$ptr_dc);
$helper->createParameterFile("$currentdir/s.params",$ptr_resource);


$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref");

#get the result value
my $ptr_res=$helper->parserResultKernel("$currentdir/output/ref");
my %res=%{$ptr_res};

print(OUTPUTFILE "config0\t$res{c_infrastructure}\t$res{c_server}\t$res{c_power}\t$res{c_network}\t$res{c_maintenance}\n");


#other configurations
for(my $i=0;$i<$nbconfig;$i++)
{
	$ptr_dc=$helper->parserDCFile($ARGV[3+$i*2]);
	$ptr_resource=$helper->parserResourceFile($ARGV[3+$i*2+1]);
	
	$helper->createParameterFile("$currentdir/dc.params",$ptr_dc);
	$helper->createParameterFile("$currentdir/s.params",$ptr_resource);
	
	$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref");
	
	#get the result value
	$ptr_res=$helper->parserResultKernel("$currentdir/output/ref");
	%res=%{$ptr_res};

	my $val=$i+1;
	print(OUTPUTFILE "config$val\t$res{c_infrastructure}\t$res{c_server}\t$res{c_power}\t$res{c_network}\t$res{c_maintenance}\n");
}

close(OUTPUTFILE);

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";

####################################################
# gnulpot call
####################################################

$helper->makeGnuplotStackedHistogram("$currentdir/$filename.txt","$currentdir","$filename","TCO Breakdown (\$\/month)",5);

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");

